@@ -12,11 +12,9 @@ from solentware_grid.gui.datarow import GRID_COLUMNCONFIGURE, GRID_CONFIGURE
 from solentware_grid.gui.datarow import WIDGET_CONFIGURE, WIDGET, ROW
 
 from pgn_read.core.constants import (
-    TAG_OPENING,
     TAG_RESULT,
-    REPERTOIRE_GAME_TAGS,
-    IFG_TAG_SYMBOL,
-    IFG_TAG_STRING_VALUE,
+    IFG_TAG_NAME,
+    IFG_TAG_VALUE,
     )
 
 from ..core.chessrecord import ChessDBrecordRepertoireTags
@@ -24,6 +22,7 @@ from .repertoiredbedit import ChessDBeditRepertoire
 from .repertoiredbdelete import ChessDBdeleteRepertoire
 from .repertoiredbshow import ChessDBshowRepertoire
 from . import constants
+from ..core.constants import TAG_OPENING, REPERTOIRE_GAME_TAGS
 
 ON_DISPLAY_COLOUR = '#eba610' # a pale orange
 
@@ -36,13 +35,17 @@ class ChessDBrowRepertoire(ChessDBrecordRepertoireTags, DataRow):
     """
     header_specification = [
         {WIDGET: tkinter.Label,
-         WIDGET_CONFIGURE: dict(text=TAG_OPENING, anchor=tkinter.W),
+         WIDGET_CONFIGURE: dict(
+             text=TAG_OPENING, anchor=tkinter.W, padx=0, pady=1,
+             font='TkDefaultFont'),
          GRID_CONFIGURE: dict(column=0, sticky=tkinter.EW),
          GRID_COLUMNCONFIGURE: dict(weight=1, uniform='player'),
          ROW: 0,
          },
         {WIDGET: tkinter.Label,
-         WIDGET_CONFIGURE: dict(text=TAG_RESULT, anchor=tkinter.W),
+         WIDGET_CONFIGURE: dict(
+             text=TAG_RESULT, anchor=tkinter.W, padx=0, pady=1,
+             font='TkDefaultFont'),
          GRID_CONFIGURE: dict(column=1, sticky=tkinter.EW),
          GRID_COLUMNCONFIGURE: dict(weight=1, uniform='result'),
          ROW: 0,
@@ -63,14 +66,18 @@ class ChessDBrowRepertoire(ChessDBrecordRepertoireTags, DataRow):
             {WIDGET: tkinter.Label,
              WIDGET_CONFIGURE: dict(
                  anchor=tkinter.W,
-                 font=constants.LISTS_OF_GAMES_FONT),
+                 font=constants.LISTS_OF_GAMES_FONT,
+                 pady=1,
+                 padx=0),
              GRID_CONFIGURE: dict(column=0, sticky=tkinter.EW),
              ROW: 0,
              },
             {WIDGET: tkinter.Label,
              WIDGET_CONFIGURE: dict(
                  anchor=tkinter.W,
-                 font=constants.LISTS_OF_GAMES_FONT),
+                 font=constants.LISTS_OF_GAMES_FONT,
+                 pady=1,
+                 padx=0),
              GRID_CONFIGURE: dict(column=1, sticky=tkinter.EW),
              ROW: 0,
              },
@@ -116,37 +123,22 @@ class ChessDBrowRepertoire(ChessDBrecordRepertoireTags, DataRow):
         Create textitems argument for ChessDBrowRepertoire instance.
 
         """
-        tags = self.value.collected_game[1]
+        tags = self.value.collected_game._tags
         return super(ChessDBrowRepertoire, self).grid_row(
             textitems=(
                 tags.get(TAG_OPENING, '?'),
                 tags.get(TAG_RESULT, '?'),
                 ),
             **kargs)
-
-    def populate_widget(self, widget, cnf=None, text=None, context=None, **kw):
-        """Wrapper for Tkinter.Text configure method for score attribute"""
-        if isinstance(widget, tkinter.Label):
-            super(ChessDBrowRepertoire, self).populate_widget(
-                widget, text=text, **kw)
-            return
-        widget.configure(cnf=cnf, **kw)
-        widget.configure(state=tkinter.NORMAL)
-        widget.delete('1.0', tkinter.END)
-        widget.insert(
-            tkinter.END,
-            '  '.join([''.join((tag, ' "', value, '"'))
-                       for tag, value in self.get_tags_display_order(text)]))
-        widget.configure(state=tkinter.DISABLED)
         
     def get_tags_display_order(self, pgn):
         """Return Tags not given their own column in display order"""
-        str_tags = []
-        for t in pgn.collected_game[0]:
-            tn = t.group(IFG_TAG_SYMBOL)
-            if tn not in REPERTOIRE_GAME_TAGS:
-                str_tags.append((tn, t.group(IFG_TAG_STRING_VALUE)))
-        return str_tags
+        tag_values = []
+        tags = self.collected_game._tags
+        for tv in sorted(tags.items()):
+            if tv[0] not in REPERTOIRE_GAME_TAGS:
+                tag_values.append(tv)
+        return tag_values
 
     def set_background_on_display(self, widgets):
         self._current_row_background = ON_DISPLAY_COLOUR
